/*
 * Copyright (c) 2013 - 2016 TDP Ltd All Rights Reserved.
 * TDP Ltd grants permission, free of charge, to any person obtaining copies
 * of this software and its associated documentation files (the "Software"),
 * to deal in the Software without restriction, including to use, copy, adapt,
 * publish, distribute, display, perform, sublicense, and sell copies of the
 * Software, subject to the following condition: You must include the above
 * copyright notice and this permission notice in all full or partial copies
 * of the Software.
 * 
 * TDP LTD PROVIDES THE SOFTWARE "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY,
 * INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGMENT. TDP LTD, THE AUTHORS OF THE SOFTWARE,
 * AND THE OWNERS OF COPYRIGHT IN THE SOFTWARE ARE NOT LIABLE FOR ANY CLAIM, DAMAGES,
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE, ARISING
 * FROM, OUT OF, OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package cz.tdp.kshield.example;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.charset.Charset;
import java.util.Locale;

import org.apache.http.Header;
import org.apache.http.HeaderIterator;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHost;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.ProtocolVersion;
import org.apache.http.StatusLine;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.HttpClientConnectionManager;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.message.BasicStatusLine;
import org.apache.http.params.HttpParams;
import org.apache.http.protocol.HttpContext;

//SONAR-IGNORE-FILE

/**
 * Used for JUnit testing and demonstration purposes
 * This implementation will always return response with given string bytes
 */
@SuppressWarnings("all")
public class MockHttpClient extends CloseableHttpClient  //NOSONAR suppress sonar warnings in Mock class
{
  /**
   * Last httpRequest - you can check this in tests
   */
  public HttpRequest httpRequest;
  
  /**
   * Last httpResponse - you can check this in tests
   */
  public MockHttpResponse httpResponse;
  
  private HttpClientConnectionManager connManager = new PoolingHttpClientConnectionManager();
  
  /**
   * @param response fake response as String
   */
  public MockHttpClient(String response) {
    setResponse(response);
  }

  /**
   * Sets new fake response as string
   * @param response
   */
  public final void setResponse(String response) {
    final byte[] bytes = response.getBytes(Charset.defaultCharset());
    final ByteArrayInputStream istream = new ByteArrayInputStream(bytes);
    
    httpResponse = new MockHttpResponse(istream, bytes);
  }
  
  public final void setStatus(int status) {
    httpResponse.setStatusCode(status);
    httpResponse.setStatus(status);
  }
  
  static final class MockHttpResponse implements CloseableHttpResponse
  {
    private final ByteArrayInputStream istream;
    private final byte[] bytes;
    private int status = HttpStatus.SC_OK;
    
    /**
     * @param istream
     * @param bytes
     */
    MockHttpResponse(ByteArrayInputStream istream, byte[] bytes) {
      this.istream = istream;
      this.bytes = bytes;
    }
    
    @Override
    public HttpEntity getEntity() {
      return new MockHttpEntity();
    }
    
    public void setStatus(int status) {
      this.status = status;
    }
    
    @Override
    public void addHeader(Header arg0) {}
    
    @Override
    public void addHeader(String arg0, String arg1) {}
    
    @Override
    public boolean containsHeader(String arg0) { return false; }
    
    @Override
    public Header[] getAllHeaders() { return null; }
    
    @Override
    public Header getFirstHeader(String arg0) { return null; }
    
    @Override
    public Header[] getHeaders(String arg0) { return null; }
    
    @Override
    public Header getLastHeader(String arg0) { return null; }
    
    @Override
    public HttpParams getParams() { return null; }
    
    @Override
    public ProtocolVersion getProtocolVersion() { return null; }
    
    @Override
    public HeaderIterator headerIterator() { return null; }
    
    @Override
    public HeaderIterator headerIterator(String arg0) { return null; }
    
    @Override
    public void removeHeader(Header arg0) {}
    
    @Override
    public void removeHeaders(String arg0) {}
    
    @Override
    public void setHeader(Header arg0) {}
    
    @Override
    public void setHeader(String arg0, String arg1) {}
    
    @Override
    public void setHeaders(Header[] arg0) {}
    
    @Override
    public void setParams(HttpParams arg0) {}
    
    @Override
    public Locale getLocale() { return null; }
    
    @Override
    public StatusLine getStatusLine() { return new BasicStatusLine(new ProtocolVersion("HTTP",1,1), status, ""); }
    
    @Override
    public void setEntity(HttpEntity arg0) {}
    
    @Override
    public void setLocale(Locale arg0) {}
    
    @Override
    public void setReasonPhrase(String arg0) throws IllegalStateException {}
    
    @Override
    public void setStatusCode(int arg0) throws IllegalStateException {}
    
    @Override
    public void setStatusLine(StatusLine arg0) {}
    
    @Override
    public void setStatusLine(ProtocolVersion arg0, int arg1) {}
    
    @Override
    public void setStatusLine(ProtocolVersion arg0, int arg1, String arg2) {}
    
    class MockHttpEntity implements HttpEntity
    {
      @Override
      public InputStream getContent() throws IOException {
        return istream;
      }
      @Override
      public void consumeContent() throws IOException {}
      @Override
      public Header getContentEncoding() { return null; }
      @Override
      public long getContentLength() { return bytes.length; }
      @Override
      public Header getContentType() { return null; }
      @Override
      public boolean isChunked() { return false; }
      @Override
      public boolean isRepeatable() { return false; }
      @Override
      public boolean isStreaming() { return false; }
      @Override
      public void writeTo(OutputStream arg0) throws IOException {}
    }

    @Override
    public void close() throws IOException {}
  }

  @Override
  public HttpParams getParams() {
    return null;
  }

  @Override
  public ClientConnectionManager getConnectionManager() {
    return null;
  }

  @Override
  public void close() throws IOException {}

  /* (non-Javadoc)
   * @see org.apache.http.impl.client.CloseableHttpClient#doExecute(org.apache.http.HttpHost, org.apache.http.HttpRequest, org.apache.http.protocol.HttpContext)
   */
  @Override
  protected CloseableHttpResponse doExecute(HttpHost target, HttpRequest request, HttpContext context)
      throws IOException, ClientProtocolException {
    httpRequest = request;
    return httpResponse;
  }
}