/*
 * Copyright (c) 2013 - 2016 TDP Ltd All Rights Reserved.
 * TDP Ltd grants permission, free of charge, to any person obtaining copies
 * of this software and its associated documentation files (the "Software"),
 * to deal in the Software without restriction, including to use, copy, adapt,
 * publish, distribute, display, perform, sublicense, and sell copies of the
 * Software, subject to the following condition: You must include the above
 * copyright notice and this permission notice in all full or partial copies
 * of the Software.
 * 
 * TDP LTD PROVIDES THE SOFTWARE "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY,
 * INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGMENT. TDP LTD, THE AUTHORS OF THE SOFTWARE,
 * AND THE OWNERS OF COPYRIGHT IN THE SOFTWARE ARE NOT LIABLE FOR ANY CLAIM, DAMAGES,
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE, ARISING
 * FROM, OUT OF, OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package cz.tdp.kshield.integration;

import cz.tdp.kshield.client.ClientMessage;
import cz.tdp.kshield.client.KShieldClientException;
import cz.tdp.kshield.client.UserInfo;


/**
 * Processes KeyShield SSO Server authentication
 */
public interface AuthenticationService
{
  /**
   * Retrieves userInfo from KeyShield SSO Server, Returns null if ip address is not validated by server, ip address cannot by validated or ip address is empty
   * Non-null return value means that ipAddr was approved by KeyShield SSO Server
   * 
   * @param ipAddr IP address (usually from servlet request)
   * @return valid user data for given ipAddr retrieved from KeyShield SSO Server or null
   * @throws IllegalStateException if service is not initialized
   */
  UserInfo createUserInfo(String ipAddr);
  
  /**
   * Sends message to another user(s) idetified by IP or FDN
   * 
   * @param from Name (Full Name) of message sender
   * @param to Message target (IP or FDN)
   * @param message Message text
   * @throws IllegalStateException if service is not initialized
   * @throws KShieldInvalidApiKeyException in case of Rejected API Key (Used API key must have been properly configured at KShield SSO Server)
   * @throws KShieldClientException in case of other communication or validation error
   */
  void sendClientMessage(String from, String to, String message) throws KShieldClientException;
  
  /**
   * Sends message to another user(s) idetified by IP or FDN
   * 
   * @param msg Client message data
   * @throws IllegalStateException if service is not initialized
   * @throws KShieldInvalidApiKeyException in case of Rejected API Key (Used API key must have been properly configured at KShield SSO Server)
   * @throws KShieldClientException in case of other communication or validation error
   */
  void sendClientMessage(ClientMessage msg) throws KShieldClientException;
  
  /**
   * Checks KeyShield SSO Server status
   * 
   * @throws IllegalStateException if service is not initialized
   * @throws KShieldInvalidApiKeyException in case of Rejected API Key
   * @throws KShieldClientException in case of other communication or validation error
   */
  void checkService() throws KShieldClientException;
  
  /**
   * Initializes Authentication service after creation
   */
  void init();
  
  /**
   * Cleanup Authentication service before destruction
   */
  void destroy();
}
