/*
 * Copyright (c) 2013 - 2016 TDP Ltd All Rights Reserved.
 * TDP Ltd grants permission, free of charge, to any person obtaining copies
 * of this software and its associated documentation files (the "Software"),
 * to deal in the Software without restriction, including to use, copy, adapt,
 * publish, distribute, display, perform, sublicense, and sell copies of the
 * Software, subject to the following condition: You must include the above
 * copyright notice and this permission notice in all full or partial copies
 * of the Software.
*
 * TDP LTD PROVIDES THE SOFTWARE "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY,
 * INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGMENT. TDP LTD, THE AUTHORS OF THE SOFTWARE,
 * AND THE OWNERS OF COPYRIGHT IN THE SOFTWARE ARE NOT LIABLE FOR ANY CLAIM, DAMAGES,
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE, ARISING
 * FROM, OUT OF, OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package cz.tdp.kshield.notification;

import org.eclipse.jetty.websocket.api.RemoteEndpoint;
import org.eclipse.jetty.websocket.api.Session;
import org.eclipse.jetty.websocket.api.annotations.OnWebSocketClose;
import org.eclipse.jetty.websocket.api.annotations.OnWebSocketConnect;
import org.eclipse.jetty.websocket.api.annotations.OnWebSocketError;
import org.eclipse.jetty.websocket.api.annotations.OnWebSocketMessage;

/**
 * Implementation of WebSocket Endpoint for Jetty Server. You must provide access to KShieldAuthManager instance
 */
public abstract class AbstractJettyKShieldEndpoint extends AbstractWebSocketEndpoint
{
  private Session session;
  private RemoteEndpoint remote;
  
  @OnWebSocketClose
  public void onWebSocketClose(int statusCode, String reason) {
    onCloseImpl();
    
    this.session = null;
    this.remote = null;
  }
  
  @OnWebSocketConnect
  public void onWebSocketConnect(Session newSession) {
    onOpenImpl();
    
    this.session = newSession;
    this.remote = this.session.getRemote();
  }
  
  @OnWebSocketError
  public void onWebSocketError(Throwable cause) {
    onErrorImpl(cause);
  }
  
  @OnWebSocketMessage
  public void onWebSocketText(String message) {
    if (this.session != null && this.session.isOpen() && this.remote != null) {
      send(onMessageImpl(message));
    }
  }
  
  @Override
  protected void send(String message) {
    if (remote != null) {
      remote.sendStringByFuture(message);
    }
  }
  
  @Override
  protected void reset() {
    session = null;
    remote = null;
    
    super.reset();
  }
}
