/*
 * Copyright (c) 2013 - 2016 TDP Ltd All Rights Reserved.
 * TDP Ltd grants permission, free of charge, to any person obtaining copies
 * of this software and its associated documentation files (the "Software"),
 * to deal in the Software without restriction, including to use, copy, adapt,
 * publish, distribute, display, perform, sublicense, and sell copies of the
 * Software, subject to the following condition: You must include the above
 * copyright notice and this permission notice in all full or partial copies
 * of the Software.
 * 
 * TDP LTD PROVIDES THE SOFTWARE "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY,
 * INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGMENT. TDP LTD, THE AUTHORS OF THE SOFTWARE,
 * AND THE OWNERS OF COPYRIGHT IN THE SOFTWARE ARE NOT LIABLE FOR ANY CLAIM, DAMAGES,
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE, ARISING
 * FROM, OUT OF, OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package cz.tdp.kshield.springsecurity3;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;

import cz.tdp.kshield.client.UserInfo;
import cz.tdp.kshield.integration.AuthenticationFactory;

/**
 * Implementation of AuthenticationFactory used in SpringAuthenticationServiceImpl based on UserDetailsService.
 * 
 * @see #createAuthenication(UserInfo)
 * @see cz.tdp.kshield.springsecurity3.SpringAuthenticationServiceImpl
 * @see org.springframework.security.core.userdetails.UserDetailsService
 */
public class SpringAuthenticationFactory implements AuthenticationFactory<Authentication>
{
  /**
   * Retrieves userDetails and creates new UsernamePasswordAuthenticationToken instance
   * 
   * @return new UsernamePasswordAuthenticationToken instance or null if authentication cannot be created
   */
  @Override
  public Authentication createAuthenication(UserInfo userInfo) {
    final UserDetails userDetails = userDetailsService.loadUserByUsername(userInfo.getUsername());
    
    if (userDetails != null) {
      try {
        return new UsernamePasswordAuthenticationToken(userDetails, userDetails.getUsername(), userDetails.getAuthorities());
      }
      catch (UsernameNotFoundException e) {
        log.warn("Unknown username", e);
      }
    }
    
    return null;
  }
  
  @Autowired
  private UserDetailsService userDetailsService;
  
  public void setUserDetailsService(UserDetailsService userDetailsService) {
    this.userDetailsService = userDetailsService;
  }

  private static final Log log = LogFactory.getLog(SpringAuthenticationFactory.class);
}