/*
 * Copyright (c) 2013 - 2016 TDP Ltd All Rights Reserved.
 * TDP Ltd grants permission, free of charge, to any person obtaining copies
 * of this software and its associated documentation files (the "Software"),
 * to deal in the Software without restriction, including to use, copy, adapt,
 * publish, distribute, display, perform, sublicense, and sell copies of the
 * Software, subject to the following condition: You must include the above
 * copyright notice and this permission notice in all full or partial copies
 * of the Software.
 * 
 * TDP LTD PROVIDES THE SOFTWARE "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY,
 * INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGMENT. TDP LTD, THE AUTHORS OF THE SOFTWARE,
 * AND THE OWNERS OF COPYRIGHT IN THE SOFTWARE ARE NOT LIABLE FOR ANY CLAIM, DAMAGES,
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE, ARISING
 * FROM, OUT OF, OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package cz.tdp.kshield.springsecurity3;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.util.Assert;

import cz.tdp.kshield.client.UserInfo;
import cz.tdp.kshield.integration.AuthenticationFactory;
import cz.tdp.kshield.integration.web.DefaultAuthenticationServiceImpl;

/**
 * Spring Security 3.0.x/3.1.x based implementation of KeyShield SSO Server AuthenticationService
 */
public class SpringAuthenticationServiceImpl extends DefaultAuthenticationServiceImpl
{
  /**
   * @param url KeyShield SSO Server url
   */
  @Autowired
  public SpringAuthenticationServiceImpl(@Value("${kshield.url:}") String url) {
    super(url);
  }
  
  @Override
  public boolean authenticate(HttpServletRequest request) {
    final Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
    
    if (authentication != null && authentication.isAuthenticated()) {
      return false;
    }
    
    return super.authenticate(request);
  }
  
  /**
   * @param request current HTTP request
   * @param userInfo User data returned by KeyShield SSO Server client
   * @return true if new authentication was created based on existing mapping
   */
  @Override
  protected boolean doAuthentication(HttpServletRequest request, UserInfo userInfo) {
    Assert.notNull(authenticationFactory, "Please provide authentication factory instance");
    
    final Authentication auth = authenticationFactory.createAuthenication(userInfo);
    
    if (auth != null && auth.isAuthenticated()) {
      SecurityContextHolder.getContext().setAuthentication(auth);
      return true;
    }
    
    // just return false if username is null
    return false;
  }

  @Autowired(required = false)
  private AuthenticationFactory<Authentication> authenticationFactory;

  /**
   * You can optionally inject custom implementation of AuthenticationFactory class
   * @param authenticationFactory - Custom AuthenticationFactory instance
   */
  public void setAuthenticationFactory(AuthenticationFactory<Authentication> authenticationFactory) {
    this.authenticationFactory = authenticationFactory;
  }
}
